import requests
import jwt as pyjwt
import base64

from conciliator.connector import Connector
from conciliator.entity import Entity
from conciliator.file import File
from conciliator.document import Document
from conciliator.page import Page
from conciliator.invoice import Invoice

API_VERSION =  "api/v0/"

app_url = "https://app.expert.conciliator.ai/"
api_url = app_url + API_VERSION
session = None

current_entity: Entity = None

def url(l):
    global app_url
    global api_url
    app_url = l
    api_url = l + API_VERSION

def connect(username, pwd, tenant=None):
    global session
    if not isinstance(session, requests.Session):
        session = requests.Session()
    params = {'callbackUrl': app_url + "?jwtResponse=${jwtResponse}"}
    r = session.get(api_url + "user/loginRedirectionUrl", params=params)

    jwt = r.text.split("=")[1]
    jwt_decoded = pyjwt.decode(jwt, verify=False)
    api_app_url = jwt_decoded["sub"]
    r = session.get(r.text)

    jwt = r.url.split("=")[1]
    credentials = base64.b64encode(("%s:%s" % (username, pwd)).encode()).decode("utf-8")
    payload = {'type': "basic", 'value': credentials, 'accountStore': {'href': api_app_url}}
    r = session.post(api_app_url + "/loginAttempts", json=payload,
                          headers={'Authorization': "Bearer %s" % jwt})
    r.raise_for_status()
    if r.text:
        # Tenant selection
        bearer = r.headers['authorization']
        org_url = r.json()['account']['href'] + '/organizations'
        r = session.get(org_url,
                             params = {'offset':0, 'limit': 10, 'q': tenant},
                             headers={'Authorization': "%s" % bearer})
        tenant_id = None
        for t in r.json()['items']:
            if t['nameKey'] == tenant:
                tenant_id = t['id']
                break
        if not tenant_id: raise ValueError('Cannot find tenant')
        bearer = r.headers['authorization']
        r = session.post(org_url + '/' + tenant_id,  headers={'Authorization': "%s" % bearer})

    jwt = r.headers['Stormpath-SSO-Redirect-Location'].split("=")[1]
    params = {'jwtResponse': jwt}
    session.get(api_url + "user/loginResult", params=params)
    return True




import json
import datetime
import typing
import inspect

class ConciliatorObj:

    def __init__(self, data):
        self.__dict__ = self.deserialize(data)

    def deserialize(self, data):

        types = {}
        for klass in inspect.getmro(self.__class__):
            if klass == ConciliatorObj: break
            types.update(typing.get_type_hints(klass))

        for k in list(data.keys()):
            if k in types:
                t = types[k]
                # date
                if t == datetime.date:
                    try:
                        data[k] = datetime.datetime.strptime(data[k] , '%Y-%m-%d').date()
                    except:
                        pass
                # datetime
                elif t == datetime.datetime:
                    try:
                        # adding trailing zeros that are removed by Conciliator API
                        data[k] = datetime.datetime.fromisoformat(
                            data[k].replace('Z', '+00:00') + '0' * (26 - len(data[k])))
                    except:
                        data.pop(k, None)
                # bool
                elif t == bool:
                    if isinstance(data[k], bool):
                        continue
                    try:
                        data[k] = (data[k].lower() == 'true')
                    except:
                        data.pop(k, None)
            else:
                data.pop(k, None)
        return data

    def serialize(self):

        types = typing.get_type_hints(self)
        data = {}

        for k in types:
            t = types[k]
            v = getattr(self, k, None)
            if t == datetime.date and v:
                try:
                    data[k] = datetime.date.isoformat(v)
                except:
                    pass
            elif t == datetime.datetime and v:
                try:
                    data[k] = datetime.datetime.isoformat(v)
                except:
                    pass
            elif v:
                data[k] = v
        return data

    def serializes(self):
        return json.dumps(self.serialize())



import conciliator
import datetime
from conciliator.conciliatorobj import ConciliatorObj

class Connector(ConciliatorObj):
    name: str
    id: str
    category: object
    config: object
    lastExecution: datetime.datetime
    linkExports: object
    numberInProcess: int
    numberLoaded: int
    numberRejected: int
    pause: bool
    status: str
    warning: object
    type: str

    def __init__(self, data):
        super().__init__(data)
        self.type = self.category['id']

    @classmethod
    def list(cls):
        connectors = []
        r = conciliator.session.get(conciliator.api_url + "connectors")
        r.raise_for_status()
        for js in r.json():
            connectors.append(Connector(js))
        return connectors

    def param(self, name):
        for conf in self.config:
            if conf['name'] == name:
                return conf['value']
        raise ValueError("Config parameter not found")

    def logs(self, nb=100):
        params = {'nbItems': nb}
        r = conciliator.session.get(conciliator.api_url + "connectors/" + self.id + "/logs", params=params)
        r.raise_for_status()
        # TODO
        return r.json()

    # zip files
    # GET 200 OK
    # https://app.expert.conciliator.ai/api/v0/connectors/8f41b5f4-1700-4334-a21d-a197fc852b84?baseUrl=https://app.expert.conciliator.ai&connectorPageUrl=https://app.expert.conciliator.ai/connectors/8f41b5f4-1700-4334-a21d-a197fc852b84
    # params: baseUrl=https://app.expert.conciliator.ai&connectorPageUrl=https://app.expert.conciliator.ai/connectors/8f41b5f4-1700-4334-a21d-a197fc852b84
    # Response:
    # "linkExports":[
    #       {
    #          "id":"1eb29a0a-2607-ca30-95a5-d062d3327d02",
    #          "name":"conciliator-isacompta-2020-11-18-14-19-06.zip",
    #          "baseURL":"https://app.expert.conciliator.ai/api/v0/connectors/8f41b5f4-1700-4334-a21d-a197fc852b84/download/1eb29a0a-2607-ca30-95a5-d062d3327d02",
    #          "downloaded":false
    #
    # Download
    # GET 200 OK
    # https://app.expert.conciliator.ai/api/v0/connectors/8f41b5f4-1700-4334-a21d-a197fc852b84/download/1eb29a0a-2607-ca30-95a5-d062d3327d02



import json
import conciliator
import datetime

class Document:

    id: str
    status: str
    totalPages: int
    name: str
    type: str
    comment: str
    fields: object
    user_tag: object
    nb_lines: int

    DATA_QUERY = [{
        "id": "goupix_invoices",
        "pagination": {
            "sort": {
                "elements": [
                    {
                        "column": "created",
                        "order": "DESC"
                    },
                    {
                        "column": "page_file_number",
                        "order": "ASC"
                    }
                ]
            },
            "windowId": {
                "firstValues": None,
                "lastValues": None
            }
        },
        "filtering": {
            "appliedFilters": {
                "entity_id": {
                    "items": [
                        {
                            "value": "<entity_id>"
                        }
                    ]
                }
            }
        }
    }]


    @classmethod
    def list(cls, query=None):
        pass
# GET
# https://app.expert.conciliator.ai/api/v0/documents?entity_id=262929ec-a9a6-424a-a90e-f1dc3e7740fc&limit=5000&invoice_type=SALE,PURCHASE

    # @classmethod
    # def from_filepages(cls, file: File):
    #     # FIXME: missing call to API /documents with arg (file_id)
    #     # 'file_name' exists but leading to errors as name is not unique
    #     # fallback to file -> page -> document
    #     l = []
    #     doc_ids = []
    #     for page in file.pages():
    #         if page.document_id in doc_ids: continue
    #         doc_ids.append(page.document_id)
    #         l.append(Document.load(page.document_id))
    #     return l

    @classmethod
    def load(cls, doc_id:str):
        r = conciliator.session.post(conciliator.api_url + 'document/' + doc_id)
        if r.status_code != 200: raise ValueError("Cannot load document from id")
        return Document(json.loads(r.text))

    def __init__(self, data):
        # TODO convert types
        self.__dict__ = data

    @property
    def pdf(self):
        r = conciliator.session.get(conciliator.api_url + 'document/' + self.id + '/download')
        if r.status_code != 200: raise ValueError("Cannot download PDF for document")
        return r.content






#     # TODO
#     def get_data(self, payload):
#         payload = '[{"id":"goupix_invoices","pagination":{"nbItems":10,"sort":{"elements":[{"column":"since","order":"DESC"}]},"windowId":{"firstValues":null,"lastValues":null}},"filtering":{"searchTerm":null,"appliedFilters":{"entity_id":{"items":[{"value":"1bd7a243-37aa-459f-a8a6-1d406361bc30"}]}}}}]'
#         return self.session.post(self.api_url + "data", data=payload,
#                                  headers={'Content-type': "application/json"}).text



import conciliator
import datetime
from conciliator.conciliatorobj import ConciliatorObj
import json

class Entity(ConciliatorObj):

    id: str
    code: str                   # code APE
    code_description: str       # description APE
    fisc_start: datetime.date
    fisc_end: datetime.date
    identifier: str
    name: str
    siret: str

    @classmethod
    def list(cls, query=None):
        l = []
        params = {'q': query}
        r = conciliator.session.get(conciliator.api_url + "entities", params=params)
        r.raise_for_status()
        for e in r.json():
            l.append(Entity(e))
        return l

    @classmethod
    def load(cls, query=None):
        es = Entity.list(query)
        if len(es) > 1: raise ValueError('Not unique result')
        if len(es) == 0: return None
        return es[0]

    def __str__(self):
        return self.identifier + ' - ' + self.name

    def set_current(self):
        conciliator.current_entity = self

    def save(self):
        r = conciliator.session.patch(conciliator.api_url + "entities",
                                       data=json.dumps([self.serialize(),]),
                                       headers={'Content-type': 'application/json'})
        r.raise_for_status()
        return True



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# To use a consistent encoding
from codecs import open
from os import path

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
      name='conciliator-python',
      
      version='0.0.2',
      
      description="Python library for the Conciliator API",
      long_description=long_description,
            long_description_content_type='text/markdown',

      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: Apache Software License",
            "Operating System :: OS Independent",
      ],

      keywords=['conciliator', 'client', 'api', 'python'],
      author='Dhatim',
      author_email='contact@dhatim.com',
      url='https://github.com/dhatim/conciliator-python',
      download_url = 'https://github.com/dhatim/conciliator-python/archive/refs/tags/0.0.2.tar.gz',
      license='Apache Software License',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      python_requires='>=3.6',
      install_requires=['requests', 'pyjwt'],
      entry_points={},
      test_suite="tests"
)



import conciliator
from conciliator.document import Document
import datetime


class Invoice(Document):

    created: str
    reference: str
    date: datetime.date
    amountInclTax: float
    amountExclTax: float
    tax: float
    supplier: str
    recipient: str
    currency: str
    exportedTimestamp: datetime.datetime
    nb_pages: int
    type_document: str
    invoice_type: str
    page_file_number: int
    lettering: str
    categories: object

    @classmethod
    def parse_response(cls, result: str):
        nb_cols = len(result['columns'])
        l = []
        for item in result['rows']:
            obj = {}
            for i in range(0, nb_cols):
                obj[result['columns'][i]['name']] = item['values'][i]['value']
            l.append(Invoice(obj))
        return l

    @classmethod
    def search(cls, query:str, nb=100, filter={}):
        data_query = Document.DATA_QUERY.copy()
        data_query[0]['pagination']['nbItems'] = nb
        data_query[0]['filtering']['searchTerm'] = query
        data_query[0]['filtering']['appliedFilters']['entity_id']['items'][0]['value'] = conciliator.current_entity.id

        for key in filter:
            # {"date": '2020-11-24'} or {"date": { "min": '2020-10-01', "max": '2020-11-01' }}
            if key == 'date':
                try:
                    d = datetime.date.isoformat(filter['date'])
                    f = { "min": d, "max": d }
                except:
                    # TODO: convert datetime.date to str
                    f = filter['date']
                data_query[0]['filtering']['appliedFilters']['date'] = f

        r = conciliator.session.post(conciliator.api_url + "data", json=data_query)
        r.raise_for_status()
        return Invoice.parse_response(r.json()[0])

    def bookentry(self):
        # GET 200 OK
        # https://app.expert.conciliator.ai/api/v0/document/1eb2e2e3-1e44-c970-ae59-38cbe272124d/entries?invoiceId=1eb2e2e3-1e44-c970-ae59-38cbe272124d


    def journal(self):
        # GET 200 OK
        # https://app.expert.conciliator.ai/api/v0/document/1eb2e2e3-1e44-c970-ae59-38cbe272124d/journal?invoiceId=1eb2e2e3-1e44-c970-ae59-38cbe272124d
        pass

    # exclude from bank reconciliation
    def exclude(self):
        r = conciliator.session.put(conciliator.api_url + "operation/invoice/" + self.id + "exclude")
        r.raise_for_status()
        return True


import json
import conciliator
from conciliator.entity import Entity
from conciliator.page import Page
from conciliator.document import Document
from conciliator.conciliatorobj import ConciliatorObj
import datetime

class File(ConciliatorObj):

    id: str
    name: str
    created: datetime.datetime
    status: str
    error: str
    user: str
    connector: str
    nb_pages: int
    entity_id: str


    @classmethod
    def list(cls, entity: Entity=None, nb=100, q=None):
        # TODO: next page with param 'last_id'
        if not entity: entity = conciliator.current_entity
        if not entity: raise ValueError('Entity is missing')
        params = {'entity_id': entity.id, 'limit': nb, 'q': q}
        r = conciliator.session.get(conciliator.api_url + "files", params=params)
        if r.status_code != 200:
            raise ValueError("Could not get file list")
        l = []
        for js in json.loads(r.text)['rowData']:
            js['entity_id'] = entity.id
            l.append(File(js))
        return l

    @classmethod
    def load(cls, entity: Entity=None, name: str=None):
        #FIXME: need to load by Id
        fs = File.list(entity, q=name)
        if len(fs) > 1: raise ValueError('Not unique result')
        if len(fs) == 0: return None
        return fs[0]

    def pages(self):
        r = conciliator.session.get(conciliator.api_url + "files/" + self.id + "/pages")
        if r.status_code != 200:
            raise ValueError("Could not get pages from file")
        l = []
        for js in json.loads(r.text):
            l.append(Page(js))
        return l

    def documents(self):
        # FIXME: need search by file_id and not filename
        params = {'entity_id': self.entity_id, 'file_name': self.name}
        r = conciliator.session.get(conciliator.api_url + 'documents/', params=params )
        l = []
        for js in json.loads(r.text)['rowData']:
            l.append(Document(js))
        return l